/*
******************************************************************************

   Demo of the LibBuf extension.

   Copyright (c) Wed Aug 29 2001 by: 
	Brian S. Julin		bri@tull.umassp.edu

  
   
This software may be redistributed under the terms of the GNU Library
(a.k.a. "Lesser") Public License -- see the file COPYING in the parent 
directory of the source code.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <ggi/ggi.h>
#include <ggi/galloc.h>
#include <ggi/buf.h>

void drawsquares(ggi_visual_t vis, ggiBuf_t z, ggiBuf_t a, 
		 int x, int y, int w, int h,
		 char *title) {
  ggi_color c;

  ggiBufSetGCForeground(a, 128);
  ggiBufSetGCBackground(a, 64);
  ggiBufSetGCForeground(z, 0);
  ggiBufSetGCBackground(z, 255);

  c.r = c.g = c.b = 65535;
  ggiSetGCForeground(vis, ggiMapColor(vis, &c));
  ggiBufSetGCForeground(z, 128);
  ggiDrawBox(vis, x + w/8, y, 6*w/8, 3*h/8);
  ggiFlush(vis);
  
  c.b = 0; c.r = 0;
  ggiSetGCForeground(vis, ggiMapColor(vis, &c));
  ggiBufSetGCForeground(z, 200);
  ggiDrawBox(vis, x, y + h/8, 3*w/8, 6*h/8);
  ggiFlush(vis);

  c.b = 65535; c.g = 0;
  ggiSetGCForeground(vis, ggiMapColor(vis, &c));
  ggiBufSetGCForeground(z, 20);
  ggiDrawBox(vis, x + 5*w/8, y + h/8, 3*w/8, 6*h/8);
  ggiFlush(vis);
  
  c.r = 65535; c.b = 0;
  ggiSetGCForeground(vis, ggiMapColor(vis, &c));
  ggiBufSetGCForeground(z, 199);
  ggiDrawBox(vis, x + w/8, y + 5*h/8, 6*w/8, 3*h/8);
  ggiFlush(vis);

  c.r = c.g = c.b = 65535;
  ggiSetGCForeground(vis, ggiMapColor(vis, &c));
  c.r = c.g = c.b = 0;
  ggiSetGCBackground(vis, ggiMapColor(vis, &c));
  ggiBufSetGCForeground(z, 0);
  ggiBufSetGCBackground(z, 0);
  ggiBufSetGCForeground(a, 255);
  ggiBufSetGCBackground(a, 255);
  ggiBufSetGCFlags(a, 0);
  ggiPuts(vis, x, y + 3*h/8, title);

}

void drawthings (ggi_visual_t vis, ggiBuf_t z,
		 int x, int y, int w, int h, int bgval, double phase,
		 char *title) {
  ggi_color c;
  int i;
  enum ggiBuf_flags tmp;

  c.r = 0; c.b = 0; c.g = 0;
  ggiSetGCForeground(vis, ggiMapColor(vis, &c));
  ggiBufSetGCForeground(z, bgval);
  ggiBufGetGCFlags(z, &tmp);
  ggiBufSetGCFlags(z, 0); /* Turn off Z clip */
  ggiDrawBox(vis, x, y, w, h);
  ggiBufSetGCFlags(z, tmp);

#define numobjs 25
  for (i = 0; i < numobjs; i++) {
    double sini, cosi;
    int bw, bh;
    sini = sin(3.14159265 * 2 * 
	       (double)((i + numobjs/3) % numobjs)/numobjs); 
    c.r = (int)(65535 + 65535 * sini)/2;
    sini = sin(3.14159265 * 2 * 
	       (double)((i + 2 * numobjs/3) % numobjs)/numobjs); 
    c.g = (int)(65535 + 65535 * sini)/2;
    sini = sin(3.14159265 * 2 * (double)i/numobjs); 
    c.b = (int)(65535 + 65535 * sini)/2;

    sini = sin(phase + 3.14159265 * 2 * (double)i/numobjs); 
    cosi = cos(phase + 3.14159265 * 2 * (double)i/numobjs);

    bw = w / 10 - sini * w / 100;
    bh = h / 10 - sini * h / 100;
    ggiSetGCForeground(vis, ggiMapColor(vis, &c));
    ggiBufSetGCForeground(z, 127 + 127 * sini);
    ggiDrawBox(vis, 
	       (int)floor(x + w/2 + w * sini/6) - bw/2, 
	       (int)floor(y + h/2 + h * cosi/3) - bh/2,
	       bw, bh);
  }

  for (i = 0; i < numobjs; i++) {
    double sini, cosi;
    int bw, bh;
    sini = sin(3.14159265 * 2 * 
	       (double)((i + numobjs/3) % numobjs)/numobjs); 
    c.r = (int)(65535 + 65535 * sini)/2;
    sini = sin(3.14159265 * 2 * 
	       (double)((i + 2 * numobjs/3) % numobjs)/numobjs); 
    c.g = (int)(65535 + 65535 * sini)/2;
    sini = sin(3.14159265 * 2 * (double)i/numobjs); 
    c.b = (int)(65535 + 65535 * sini)/2;

    ggiSetGCForeground(vis, ggiMapColor(vis, &c));
    ggiBufSetGCForeground(z, 127 - 127 * sini);

    sini = sin(phase + 3.14159265 * 2 * (double)i/numobjs); 
    cosi = cos(phase + 3.14159265 * 2 * (double)i/numobjs);

    bw = i * w / numobjs;
    bh = i * h / numobjs;
    ggiDrawHLine(vis, (w - bw) / 2, (h - h * i / numobjs) / 2, bw);
    ggiDrawHLine(vis, (w - bw) / 2, (h + h * i / numobjs) / 2, bw);
    ggiDrawVLine(vis, (w - w * i / numobjs) / 2, (h - bh) / 2, bh);
    ggiDrawVLine(vis, (w + w * i / numobjs) / 2, (h - bh) / 2, bh);
  }


}

int main(void)
{
	ggiGA_resource_list reqlist;
	ggi_visual_t vis;
	ggiBuf_t z, a, bufbunch[2];
	ggi_mode mode;
	enum ggiBuf_flags postdiv;
	int i, j;
	double phase;

	reqlist = NULL; /* Needed for Create* functions */

	/* Initialize LibGGI */
	if (ggiInit() != 0) {
		fprintf(stderr, "Unable to initialize LibGGI\n");
		exit(1);
	}
	
	/* Initialize LibBuf extension */
	if (ggiBufInit() != 0) {
		ggiPanic("Unable to initialize LibBuf extension\n");
	}

	/* Open the default visual */
	if ((vis = ggiOpen(NULL)) == NULL) {
		ggiPanic("Unable to open default visual\n");
	}
	
	/* Turn on asynchronous mode (which should always be used) */
	ggiSetFlags(vis, GGIFLAG_ASYNC);
	
	/* Set the default mode */
	if (ggiSetSimpleMode(vis, GGI_AUTO, GGI_AUTO, GGI_AUTO, GT_AUTO) < 0) {
		ggiPanic("Unable to set default mode\n");
	}

	ggiGetMode(vis, &mode);
	
	/* Attach the LibBuf extension to the visual */
	if (ggiBufAttach(vis, &reqlist) < 0) {
		ggiPanic("Unable to attach LibBuf extension to visual\n");
	}

	z = ggiBufCreateZ(vis, NULL, GA_STORAGE_SWAP,
			      GT_CONSTRUCT(8, GT_GREYSCALE, 8), BUF_CMP_LTE);
	a = ggiBufCreateAlpha(vis, NULL, GA_STORAGE_SWAP, 
			      GT_CONSTRUCT(8, GT_GREYSCALE, 8), 0);
	if (z == NULL) {
		fprintf(stderr, "Could not create Z buffer,"
			"skipping Z tests.\n");
		goto alpha;
	}

	ggiBufCladVis(vis, 1, &z);

	phase = 0;
	while (1) {
		ggiBufSetGCBackground(z, 255);
		ggiBufClearBuffer(z);
		ggiBufSetGCFlags(z, BUF_CMP_LTE);
		drawthings (vis, z, 0, 0, mode.visible.x, 
			    mode.visible.y, 255, phase, ""); 
		ggiFlush(vis);
		phase += 0.01;
		if (phase > 3.1415) break;
		usleep(100000);
	}

	(void) ggiGetc(vis);

	ggiBufStrip(vis);

 alpha:
	if (a == NULL) {
		fprintf(stderr, "Could not create Alpha buffer,"
			"skipping Alpha tests.\n");
		goto out;
	}

	/* TODO: Alpha-only demos */

	if (z == NULL) goto out;

	bufbunch[0] = z;
	bufbunch[1] = a;
	ggiBufCladVis(vis, 2, bufbunch);

	ggiBufSetGCFlags(z, BUF_CMP_LTE);

	ggiBufSetGCForeground(a, 0);
	ggiBufSetGCBackground(a, 0);
	ggiBufSetGCForeground(z, 255);
	ggiBufSetGCBackground(z, 255);
	ggiBufClearBuffer(a);
	ggiBufClearBuffer(z);

	postdiv = BUF_BLEND_POSTDIV;
	while (1) {
	ggiBufSetGCForeground(a, 255);
	ggiBufSetGCFlags(a, 0);
	ggiBufSetGCForeground(z, 0);
	for (i = 0; i < mode.visible.x; i++) {
	  ggi_color c;
	  c.r = c.g = c.b = i % 10 * 6553;
	  ggiSetGCForeground(vis, ggiMapColor(vis, &c));
	  ggiDrawVLine(vis, i, 0, mode.visible.y);
	}
	ggiBufSetGCBackground(z, 255);
	ggiBufClearBuffer(z);

	for (i = 0; i < mode.visible.y; i++) {
	  if (i % 9) 
	    ggiBufConst2Buf(a, 0, i, 0, ((i % 9) << 5) - 1, mode.visible.x, 1);
	  else 
	    ggiBufConst2Buf(a, 0, i, 0, 0, mode.visible.x, 1);
	}


#define drawblendsquares(blend, title) 				\
	  ggiBufSetGCFlags(a, blend | postdiv);	\
	  drawsquares(vis, z, a, (i * mode.visible.x)/11, 	\
		      (j * mode.visible.y)/11, 			\
		      mode.visible.x/12, mode.visible.y/13, title);

	i = 1; j = 0;
	
	drawblendsquares(BUF_BLEND_OVER,    " OVER  ");		i++;
	drawblendsquares(BUF_BLEND_UNDER,   " UNDER ");		i++;
	drawblendsquares(BUF_BLEND_XOR,     "  XOR  ");	        i++;
	drawblendsquares(BUF_BLEND_PLUS,    " PLUS  ");		i++;
	drawblendsquares(BUF_BLEND_SRC_IN,  "SRC_IN ");	        i++;
	drawblendsquares(BUF_BLEND_DST_IN,  "DST_IN ");		i++;
	drawblendsquares(BUF_BLEND_SRC_OUT, "SRC_OUT");	        i++;
	drawblendsquares(BUF_BLEND_DST_OUT, "DST_OUT");	        i++;
	drawblendsquares(BUF_BLEND_TOP,     "  TOP  ");		i++;
	drawblendsquares(BUF_BLEND_BOTTOM,  "BOTTOM ");		i++;

	i = 0; j = 1;

#undef drawblendsquares
#define drawblendsquares(blend, title) 				\
	  ggiBufSetGCFlags(a, ((blend | postdiv)	\
		      << BUF_ALTBLEND_SHIFT) | BUF_BLEND_ALT);	\
	  drawsquares(vis, z, a, (i * mode.visible.x)/11, 	\
		      (j * mode.visible.y)/11, 			\
		      mode.visible.x/12, mode.visible.y/13, title);

	drawblendsquares(BUF_BLEND_OVER,    " OVER  ");		j++;
	drawblendsquares(BUF_BLEND_UNDER,   " UNDER ");		j++;
	drawblendsquares(BUF_BLEND_XOR,     "  XOR  ");	        j++;
	drawblendsquares(BUF_BLEND_PLUS,    " PLUS  ");		j++;
	drawblendsquares(BUF_BLEND_SRC_IN,  "SRC_IN ");	        j++;
	drawblendsquares(BUF_BLEND_DST_IN,  "DST_IN ");		j++;
	drawblendsquares(BUF_BLEND_SRC_OUT, "SRC_OUT");	        j++;
	drawblendsquares(BUF_BLEND_DST_OUT, "DST_OUT");	        j++;
	drawblendsquares(BUF_BLEND_TOP,     "  TOP  ");		j++;
	drawblendsquares(BUF_BLEND_BOTTOM,  "BOTTOM ");		j++;

#undef drawblendsquares
#define drawblendsquares(blend1, blend2) 				     \
	  ggiBufSetGCFlags(a, ((blend2 | postdiv)                  \
                                      << BUF_ALTBLEND_SHIFT)                 \
			       | blend1 | postdiv | BUF_BLEND_ALT);\
	  drawsquares(vis, z, a, (i * mode.visible.x)/11, 		     \
		      (j * mode.visible.y)/11, 			             \
		      mode.visible.x/12, mode.visible.y/12, "");

#define drawcolumn(blend1) \
	j = 1;			\
	drawblendsquares(blend1, BUF_BLEND_OVER);		j++;\
	drawblendsquares(blend1, BUF_BLEND_UNDER);		j++;\
	drawblendsquares(blend1, BUF_BLEND_XOR);		j++;\
	drawblendsquares(blend1, BUF_BLEND_PLUS);		j++;\
	drawblendsquares(blend1, BUF_BLEND_SRC_IN);		j++;\
	drawblendsquares(blend1, BUF_BLEND_DST_IN);		j++;\
	drawblendsquares(blend1, BUF_BLEND_SRC_OUT);		j++;\
	drawblendsquares(blend1, BUF_BLEND_DST_OUT);		j++;\
	drawblendsquares(blend1, BUF_BLEND_TOP);		j++;\
	drawblendsquares(blend1, BUF_BLEND_BOTTOM);		j++;

	i = 1;
	drawcolumn(BUF_BLEND_OVER);		i++;
	drawcolumn(BUF_BLEND_UNDER);		i++;
	drawcolumn(BUF_BLEND_XOR);		i++;
	drawcolumn(BUF_BLEND_PLUS);		i++;
	drawcolumn(BUF_BLEND_SRC_IN);		i++;
	drawcolumn(BUF_BLEND_DST_IN);		i++;
	drawcolumn(BUF_BLEND_SRC_OUT);		i++;
	drawcolumn(BUF_BLEND_DST_OUT);		i++;
	drawcolumn(BUF_BLEND_TOP);		i++;
	drawcolumn(BUF_BLEND_BOTTOM);		i++;

	if (!postdiv) break;
	postdiv = 0;

	/* Wait for any keypress. 
	 * This is a blocking call which returns a 16 bit wide unicode
	 * character
	 */
	(void) ggiGetc(vis);


	}


 out:
	(void) ggiGetc(vis);

	/* Detach the LibBuf extension from visual */
	ggiBufDetach(vis);

	/* Close visual */
	ggiClose(vis);

	/* Deinitialize LibBuf extension */
	ggiBufExit();

	/* Deinitialize LibGGI */
	ggiExit();

	return 0;
}


#if 0
  /* Code fragment For eyeball consistancy testing */
  ggiBufConst2Buf(z, x, y, 0, bgval, w, h);
  c.r = 0; c.b = 0; c.g = 0;
  ggiSetGCForeground(vis, ggiMapColor(vis, &c));
  ggiBufSetGCForeground(z, bgval);
  ggiDrawBox(vis, x, y, w, h);

  for (i = 0; i < w; i++) {
    ggiBufConst2Buf(z,i,y,0, i % 8, 1, h);
  }
  c.r = 0; c.b = 0; c.g = 65535;
  ggiSetGCForeground(vis, ggiMapColor(vis, &c));
  for (i = 0; i < h; i++) {
    ggiBufSetGCForeground(z, i % 8);
    ggiDrawHLine(vis, 0, i, 64 - i / 16);
  }
  ggiFlush(vis);

  (void) ggiGetc(vis);
#endif




